# Anna Agents-Oriented Testing Setup

## Agents configuration

```
$> python3 loader.py -h
usage: loader.py [-h] -f FILE

Anna Agents-Oriented Testing Setup Loader

optional arguments:
  -h, --help            show this help message and exit
  -f FILE, --file FILE  Agents yaml configuration file
```

The python script 'loader.py' will preload and configure all the needed agents.
Also, ADML ones will start with the configured endpoints, ready for work.

```
python3 loader.py --file <tests directory>/agents.yml
```

See example at './tests.example/agents.yml'.

It is recommended to have that file within the tests directory compatible with such layout.
Anyway, you could have a common agents definition for multiple directories wherever you want to store them.
Next procedure (execution) will get a working directory which will be the parent for all the tests processed.

Also, you could load several agents definitions.

The format for agents file specification is:

For KAFKA:

```
<node label>:
  description: <description>
  template: KAFKA
  topic: <kafka topic>
```

For ADML (diameter):

```
<node label>:
  description: <description>
  template: ADML
  application_id: <value>
  dictionary: <xml dictionary path (absolute or relative to ./agents/ADML)>
  type: client | server
  origin_host: <origin host>
  origin_realm: <origin realm>
  address: <ip/server:port[,ip2/server2:port,...]>
```

For HTTPMOCK:

```
<node label>:
  description: <description>
  template: HTTPMOCK
  address: <ip/server:port>
```

Example:

```
AF:
  description: AF Diameter Rx Node
  template: ADML
  application_id: 16777236
  dictionary: stacks/DictionaryRx.16777236.xml
  type: client
  origin_host: machine.source.server.realm.com
  origin_realm: source.server.realm.com
  address: 127.0.0.1:3868

SMPC:
  description: SMPC N7 Node
  template: KAFKA
  topic: n7topic

DB:
  description: database
  template HTTPMOCK
  address: dbnode:9090
```


## Test launcher

```
$> python3 launcher.py -h
usage: launcher.py [-h] -t TESTS_DIR [-k] [-s] [-i] [-d]                                                                                        [1/708]

Anna Agents-Oriented Testing Setup Launcher

optional arguments:
  -h, --help            show this help message and exit
  -t TESTS_DIR, --tests-dir TESTS_DIR
                        Tests parent directory where to find .yml files (from
                        the next directories level)
  -k, --keep-list-if-exists
                        Keeps intact the list of test cases (<test-
                        dir>/launcher.list), creates it if missing
  -s, --stop-adml-at-the-end
                        At the end, ADML keeps running to ease debugging. You
                        could force stop with this option
  -i, --interactive     Interactive execution to ease debugging of test cases
  -d, --dry-run         Used to test and debug provision, no execution is
                        launched
  -p IP_LIMIT, --ip-limit IP_LIMIT
                        In-Progress limit is the number of coexisting In-
                        Progress State test cases. Defaults to 1 (sequential),
                        -1 would be 'no limit').
  -r TTPS, --ttps TTPS  Rate of test cases launched (test ticks per second).
                        By default 50 (recommended for monothread version).
```

The python script 'launcher.py' will execute the test cases under the provided directory.
Test cases at the directory provided itself will be ignored (searchs are performed at second level in order to avoid catching agents definitions).
Then we recommend this structure:

```
<tests directory>/agents.yml
<tests directory>/<test suite>/.../<test case>.yml
<tests directory>/<test suite(i)>/.../<dir(j)>/.../<test case(k)>.yml
```

The organization of test cases, chapters, test suites, or whatever desired concept is completely free for user preferences. Anyway, we recommend the use of numeric prefixes to control the default order of execution (alphabetical order).

To execute the tests, just provide the global parent directory.
NO TEST CASES SHALL BE LOCATED THERE BY CONVENTION (indeed, you could have them, but they would be ignored).

```
python3 launcher.py --tests-dir <tests directory>
```

The procedure will look for the file ```'<tests directory>'/launcher.list'``` in order to plan the execution.
That list will be created/refreshed always, except if you pass the option '-k|--keep-list-if-exists' (in case you want to play with edited lists).
The algorithm to create the list is based in a recursive search of yaml files in alphabetical order under the tests directory provided, in a similar way than the output of:

```
find <tests directory> -mindepth 2 -name "*.yml" | sort -t'/'
```

### Interactive mode

One interesting feature from AOTS is the possibility to interact test cases even to deep level (step by step).
You could enable the interactive mode by mean:

```
python3 launcher.py --tests-dir <tests directory> --interactive
```

A menu will be shown after test cases programming to give you a complete control of the ADML operations:

```
    MAIN INTERACTIVE MENU
    =====================
    (prefix option with 'h' to get detailed help)

    General:
    0.  Exit/Quit

    Position:
    1.  Go to
    2.  Look

    Test cases execution:
    3.  Start with test rate
    4.  Start next N test cases
    5.  In-progress limit

    Low level execution: test case steps
    6.  Execute next N steps

    Status & cycling:
    7.  Reset
    8.  Pool repeats
    9.  Collect
    10. Auto reset
    11. Reports configuration
```

### User-defined lists

You could edit such list and remove or comment (#) entries (each line is a test case) in order to "disable" such test cases.
You could also mess the order with something like:

```
sort -R <tests directory>/launcher.list
```

Don't forget to pass the option '-k|--keep-list-if-exists' or the list will be overwritten during launcher procedure execution.

### Test cases

Files used by parsers within the test cases (xml diameter messages, json kafka ones, etc.), are relative to the test case file path location, anyway you could set absolute paths for the stuff used.

Recomendations:

* Alternative 1: create a directory called ```'$(basename <test case file> .yml)'```
  in order to keep order:

```
     <dir>/tc1-initial-with-ipv4.yml
     <dir>/tc1-initial-with-ipv4/AAA.xml
     <dir>/tc1-initial-with-ipv4/smpc-consumed.json
     <dir>/tc2-initial-with-ipv6.yml
     <dir>/tc2-initial-with-ipv6/AAA.xml
     <dir>/tc2-initial-with-ipv6/smpc-consumed.json
     ...
```

* Alternative 2: create a unique directory for the definition and all the test case stuff:

```
     <dir>/tc1/initial-with-ipv4.yml
     <dir>/tc1/AAA.xml
     <dir>/tc1/smpc-consumed.json
     <dir>/tc2/initial-with-ipv6.yml
     <dir>/tc2/AAA.xml
     <dir>/tc2/smpc-consumed.json
     ...
```

The last one seems to be better and cleaner alternative.

### Test case definition

A test case is a yaml list of steps. Each step is a dictionary object with mandatory keys called 'action' and 'arguments'.
An action could be generic (no agent id specified) or agent-related.
Arguments depends on the type of action and are documented at ```'agents/<template>/ACTIONS.md'``` file.
Generic actions are documented below ([Generic actions](#generic-actions)).

```
- action: [<agent id>/]<action id>
  arguments:
    <argument 1>: <value 1>
    ...
    <argument m>: <value m>
  ...
```

Example:

```
# Step 1: Establish a timeout for the whole test case
- action: timeout_ms
  arguments:
    value: 1000

# Step 2: Clean kafka message bus for the SMPC topic
- action: SMPC/admin
  arguments:
    operation: clean_topic

# Step 3: Send the AA-Request to the server
- action: AF/send_xml_to_entity
  arguments:
    xml: AAR.xml

# Step 4: Wait for AA-Answer for the previous request sent
- action: AF/wait_xml_from_entity
  arguments:
    xml: AAA.xml
    answers_to: 3

# Step 4: Check the message bus for a kafka notify reception
- action: SMPC/consume_json
  arguments:
    json: expected.json
    auto_offset_reset: earliest

```

It is good to comment each step with the position number (1..N). Take into account that lists are processed in order. Some steps could reference to other steps and this position could be useful for that purpose (see step 4 in the example above).

### Generic actions

#### sh_command

Sets a shell command script execution.

Arguments: value
* value: command and arguments

Example:
```
- action: sh_command
  arguments:
    value: cat AAR.xml
```

#### timeout_ms

Sets timeout (milliseconds) to complete the whole test case, or get a Failed state if not completed.
This could be configured at any step point, where will be started as time measuring reference.
It is recommended in order to ensure that test will not be in-progress eternally.
The launcher executor will calculate 15 seconds per test case by default just in case this
timeout feature is not used.

Arguments: value
* value: numeric value for the lapse of time in milliseconds.

Example:
```
- action: timeout_ms
  arguments:
    value: 3500
```

#### delay_ms

Sets delay (milliseconds) to sleep the test case at the corresponding delay step.

Arguments: value
* value: numeric value for the lapse of time in milliseconds.

Example:
```
- action: delay_ms
  arguments:
    value: 200
```

#### ip_limit

Controller In-Progress limit. A value of 1 means that only 1 test case with
status "in progress" is allowed. That is to say, ip_limit=1 is the same as
sequential execution of test cases.

This value can be set for the whole launcher execution by mean '-p|--ip-limit'
argument, but anyway, the value set in a test case definition has priority
because it will remain for the following test cases executed if the action
is not used to update the value again. Summing up, the command line for
launcher only sets the initial ip-limit value (1 by default).

Also we can restore the command-line value in any moment just configuring
"launcher" for this action value.

A typical use case for this is when a chapter or a whole test suite need a
initial (or intermediate) administration step to do global actions which
could impact in the real test cases if everything is executed in parallel.
For example, remove kafka bus or create first dummy message, could impact
in the real test cases which are consuming data from the kafka message bus.
Then, we will define a first test-suite case with the proper administrative
operations setting ip-limit to 1, doing needed operations and updating this
limit again (to -1, no limit, or any other). The reserved word "launcher"
helps to restore the value which the user provided through the launcher
procedure (in this way, if user wanted sequential, the tests will be
sequentially executed).

To ensure that an intermediate test case is the only executed we should
first set the ip-limit to 1, then perform a delay step with the maximum
timeout of the suite test cases, and then, perform the administrative
operations. Finally, as it is unpredictable to know which test case is the
following, in the same administrative test case, we end defining another
ip_limit action to restore the parallel value (ip limit > 1 or "launcher").
Also, those administrative operations should be blocking until completed.

Arguments: value
* value: in progress limit amount (-1: no limit, 0: stops the ttps clock). Reserved 'launcher' sets the command-line launcher received value.

Example:
```
- action: ip_limit
  arguments:
    value: 1
```

#### modify_xml_avp_data

Replace operations over ANNA diameter message xml representation.
Replaces data (or hex-data) fields for specific avps before processing.
XPath is used to select the node(s) to be processed. The value provided
will replace the data or hex-data field. A list is used to allow one or
more replace operations over the same xml file.

Arguments: xml, new_xml, xpath_value_list
* xml: the diameter message in xml format. Could be a symlink to common template.
* new_xml: new diameter message in xml format. Should be the one referred at test case.
* xpath_value_list: list of items with fields: xpath, value

Example:
```
- action: modify_xml_avp_data
  arguments:
    xml: AAR.xml.template
    new_xml: AAR.xml
    xpath_value_list:
    - xpath: "avp[@name='Session-Id']"
      value: "mytest;afNodeHostname.nodeHostRealm.com;1;005"
    - xpath: "avp[@name='Framed-IP-Address']"
      value: "c52a0b20"
```

Important note: better don't use for diameter messages used in wait conditions.
The reason is that wait conditions acts using the xml content as regular
expression to be matched with serialized message received at the endpoint.
The thing is that diameter xml representation compared with python parser
representation will be probably different and regular expression won't be
valid to match. Summing up, this kind of messages must be statically
created with the expected content. Anyway, fields like hop-by-hop, end-to-end
and Origin-State-Id AVP values, will be automatically matched regardless its
value (regular expression is internally modified to replace them by '[0-9]+').

#### modify_json_key_value

Replaces value fields for specific keys before processing.

Arguments: json, new_json, kpath_value_list
* json: the json file. Could be a symlink to common template.
* new_json: new json file. Must be the one referred at test case.
* kpath_value_list: list of items with fields: kpath, value

The 'key path' concept (kpath field) consist in json node location
by mean a dot-separated string: <node1>[.<node2>]..[.<nodeN>].

Example:
```
- action: modify_json_key_value
  arguments:
    json: notify.json.template
    new_json: notify.json
    kpath_value_list:
    - kpath: "session.smSessionId"
      value: "the_sm_session_id"
```

#### system_cmd

Launchs an inline shell command or a user script file.

Arguments: [shell, file, file_parameters]
* shell: inline shell command and parameters.
* file: script or procedure defined in a test case file
* file_parameters: script file parameters (empty by default)

Example:
```
- action: system_cmd
  arguments:
    shell: echo "this is a test"

- action: system_cmd
  arguments:
    shell: rdate remote_host

- action: system_cmd
  arguments:
    file: check_consumer_step.py
    file_parameters: 2
```

## Environment variables

Agents directory, where templates ADML and KAFKA are located, could be accessible through environment variable 'AGENTS_DIR'.
For example, you could use it for an system_cmd action:

Example:
```
- action: system_cmd
  arguments:
    value: check_consumption.sh $AGENTS_DIR/KAFKA/SMPC-consumer.sh.output
```

## Logs generated

After launcher execution, a '<tests directory>.logs' directory is created within test directory processed.
This is an example of logs created by AOTS:

```
drwxrwxr-x 2 vagrant vagrant  4096 Aug 18 21:24 traffic
-rw-rw-r-- 1 vagrant vagrant 16065 Aug 18 21:24 junit.xml
-rw-rw-r-- 1 vagrant vagrant 15166 Aug 18 21:24 tests.summary
-rw-rw-r-- 1 vagrant vagrant 10747 Aug 18 21:24 tests.oam
-rw-rw-r-- 1 vagrant vagrant   697 Aug 18 21:24 tests.stats
-rw-rw-r-- 1 vagrant vagrant 83839 Aug 18 21:24 adml.context
drwxrwxr-x 2 vagrant vagrant  4096 Aug 18 21:24 counters
drwxrwxr-x 2 vagrant vagrant  4096 Aug 18 21:24 test-reports
drwxrwxr-x 2 vagrant vagrant  4096 Aug 18 21:24 debug
```

* traffic: folder containing traffic logs, for example diameter interfaces flow information.
* junit.xml: JUnit report for the testsuite execution.
* tests.summary: ADML director tests summary in xml format.
* tests.oam: ADML director Operation & Maintenance information (counters)
* tests.stats: ADML director statistic module information (average, deviation, max & min values for any of the statistic concepts registered, for example processing time in every interface, messages size, etc.).
* adml.context: context information in xml format for the ADML director process.
* counters: counters information, for example diameter events.
* test-reports: ADML tests reports in xml format.
* debug: debug information (ADML traces, yml test case files list, relation between ids and descriptions, ADML operations dumps, etc.).

