// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_comm_Variable_hpp
#define anna_comm_Variable_hpp

#include <anna/core/util/Variable.hpp>
#include <anna/core/DataBlock.hpp>

#include <anna/comm/functions.hpp>

namespace anna {

namespace comm {

class CompatCodec;

/**
   Clase para guardar la informacion asociada a un anna::comm::Codec.

   \warning Esta clase no establece proteccion ante accesos concurrentes
*/
class Variable : public anna::Variable {
public:
  /**
     Constructor para inicializar una instancia de tipo cadena.
    \param id Identificador logico de esta variable
    @param name Nombre logico que recibe este variable.
    @param value Referencia a la variable que estamos recubriendo con esta instancia.
  */
  Variable(const short int id, const char* name, std::string& value) :
    anna::Variable(name, value),
    a_id(id), a_precode(true) {;}

  /**
  * Constructor para inicializar una instancia de type ENTERO.
  *
    \param id Identificador logico de esta variable
  * @param name Nombre logico que recibe este variable.
  * @param value Referencia a la variable que estamos recubriendo con esta instancia.
  */
  Variable(const short int id, const char* name, int& value) :
    anna::Variable(name, value),
    a_id(id), a_precode(true) {;}

  /**
  * Constructor para inicializar una instancia de type ENTERO_LARGO.
  *
    \param id Identificador logico de esta variable
  * @param name Nombre logico que recibe este variable.
  * @param value Referencia a la variable que estamos recubriendo con esta instancia.
  */
  Variable(const short int id, const char* name, S64& value) :
    anna::Variable(name, value),
    a_id(id), a_precode(true) {;}

  /**
  * Constructor para inicializar una instancia de type BOOLEAN.
  *
    \param id Identificador logico de esta variable
  * @param name Nombre logico que recibe este variable.
  * @param value Referencia a la variable que estamos recubriendo con esta instancia.
  */
  Variable(const short int id, const char* name, bool& value) :
    anna::Variable(name, value),
    a_id(id), a_precode(true) {;}

  /**
  * Constructor para inicializar una instancia de type BLOQUE_MEMORIA.
  *
    \param id Identificador logico de esta variable
  * @param name Nombre logico que recibe este variable.
  * @param value Referencia a la variable que estamos recubriendo con esta instancia.
  */
  Variable(const short int id, const char* name, DataBlock& value) :
    anna::Variable(name, value),
    a_id(id), a_precode(true) {;}

  /**
  * Constructor para inicializar una instancia de tipo float.
  *
    \param id Identificador logico de esta variable
  * @param name Nombre logico que recibe este variable.
  * @param value Referencia a la variable que estamos recubriendo con esta instancia.
  */
  Variable(const short int id, const char* name, float& value) :
    anna::Variable(name, value),
    a_id(id), a_precode(true) {;}

  /**
  * Constructor para inicializar una instancia de tipo float.
  *
    \param id Identificador logico de esta variable
  * @param name Nombre logico que recibe este variable.
  * @param value Referencia a la variable que estamos recubriendo con esta instancia.
  */
  Variable(const short int id, const char* name, double& value) :
    anna::Variable(name, value),
    a_id(id), a_precode(true) {;}

  /**
  * Constructor para inicializar una instancia de tipo float.
  *
    \param id Identificador logico de esta variable
  * @param name Nombre logico que recibe este variable.
  * @param value Referencia a la variable que estamos recubriendo con esta instancia.
  */
  Variable(const short int id, const char* name, comm::CompatCodec& value) :
    anna::Variable(name, (void*) &value),
    a_id(id), a_precode(true) {;}

  /**
     Devuelve el identificador logico de esta variable.
     \return el identificador logico de esta variable.
  */
  short int getId() const throw() { return a_id; }

private:
  const short int a_id;
  DataBlock a_precode;

  const DataBlock& codec() const throw(RuntimeException) {
    return (a_precode.isEmpty() == false) ? a_precode : precodec();
  }
  const DataBlock& precodec() const throw(RuntimeException) {
    char aux [sizeof(short int)];
    const_cast <comm::Variable*>(this)->a_precode.append(comm::functions::codeShort(aux, a_id), sizeof(short int));
    const_cast <comm::Variable*>(this)->a_precode += (char) getType();
    return a_precode;
  }

  friend class CompatCodec;
};

}

}

#endif

