// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_diameter_helpers_defines_hpp
#define anna_diameter_helpers_defines_hpp


// Local
#include <anna/diameter/defines.hpp>
#include <anna/config/defines.hpp>


/**
* Define VENDORID__<context> with S32 value
*/
#define anna_diameter_helpers_define_vendor(vendorname,code)\
   /** vendor #vendorname */\
   static const S32 VENDORID__##vendorname = code

/**
* Define APPID__<context> with S32 value (top = 4294967295)
*/
#define anna_diameter_helpers_define_appid(appname,code)\
   static const S32 APPID__##appname = code

/**
* Define AVPID__<avpname> for any <context> with AvpId value
*/
#define anna_diameter_helpers_define_avp_in_context(avpname,code,context)\
   static const AvpId AVPID__##avpname(code,VENDORID__##context)

/**
* Define AVPID__<avpname> and AVPVALUES__<avpname> for any <context> with AvpId value and static struct
*/
#define anna_diameter_helpers_define_avp_with_values_in_context(avpname,code,context)\
   anna_diameter_helpers_define_avp_in_context(avpname,code,context);\
   struct AVPVALUES__##avpname\
 
/**
* Define COMMANDID__<commandname> for requests with CommandId value
*/
#define anna_diameter_helpers_define_command_request(commandname,code)\
   static const CommandId COMMANDID__##commandname(code,true)

/**
* Define COMMANDID__<commandname> for answers with CommandId value
*/
#define anna_diameter_helpers_define_command_answer(commandname,code)\
   static const CommandId COMMANDID__##commandname(code,false)


namespace anna {

namespace diameter {

namespace helpers {

/** Generic AVP */
static const AvpId AVPID__AVP(0, 0);

/** Reserved Generic COMMAND */
//static const AvpId COMMANDID__COMMAND(0, 0);

/**
* Vendors
* http://www.iana.org/assignments/enterprise-numbers
*/
anna_diameter_helpers_define_vendor(radius, 0);      // VENDORID__radius
anna_diameter_helpers_define_vendor(base, 0);        // VENDORID__base
anna_diameter_helpers_define_vendor(dcca, 0);        // VENDORID__dcca
anna_diameter_helpers_define_vendor(ietf, 0);        // VENDORID__ietf
anna_diameter_helpers_define_vendor(etsi, 13019);    // VENDORID__etsi
anna_diameter_helpers_define_vendor(tgpp, 10415);    // VENDORID__tgpp
anna_diameter_helpers_define_vendor(nokia, 94);      // VENDORID__nokia
anna_diameter_helpers_define_vendor(ericsson, 193);  // VENDORID__ericsson
anna_diameter_helpers_define_vendor(huawei, 2011);   // VENDORID__huawei
anna_diameter_helpers_define_vendor(tid, 5189);      // VENDORID__tid
anna_diameter_helpers_define_vendor(tme, 27640);     // VENDORID__tme

/**
* Application identifiers
* http://www.iana.org/assignments/aaa-parameters/aaa-parameters.xhtml
*/
anna_diameter_helpers_define_appid(Diameter_Common_Message, 0);                          // APPID__Diameter_Common_Message
anna_diameter_helpers_define_appid(NASREQ, 1);                                           // APPID__NASREQ
anna_diameter_helpers_define_appid(Mobile_IPv4, 2);                                      // APPID__Mobile_IPv4
anna_diameter_helpers_define_appid(Diameter_Base_Accounting, 3);                         // APPID__Diameter_Base_Accounting
anna_diameter_helpers_define_appid(Diameter_Credit_Control, 4);                          // APPID__Diameter_Credit_Control
anna_diameter_helpers_define_appid(Diameter_EAP, 5);                                     // APPID__Diameter_EAP
anna_diameter_helpers_define_appid(Diameter_Session_Initiation_Protocol_Application, 6); // APPID__Diameter_Session_Initiation_Protocol_Application
anna_diameter_helpers_define_appid(Diameter_Mobile_IPv6_IKE, 7);                         // APPID__Diameter_Mobile_IPv6_IKE
anna_diameter_helpers_define_appid(Diameter_Mobile_IPv6_Auth, 8);                        // APPID__Diameter_Mobile_IPv6_Auth
anna_diameter_helpers_define_appid(Diameter_QoS_Application, 9);                         // APPID__Diameter_QoS_Application
// etc.
anna_diameter_helpers_define_appid(3GPP_Rx, 16777236);                                   // APPID__3GPP_Rx
anna_diameter_helpers_define_appid(3GPP_Gx, 16777238);                                   // APPID__3GPP_Gx
anna_diameter_helpers_define_appid(3GPP_Sy, 16777302);                                   // APPID__3GPP_Sy

// much more...

}
}
}


#endif

