// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_xml_DocumentMemory_hpp
#define anna_xml_DocumentMemory_hpp

#include <anna/config/defines.hpp>
#include <anna/xml/Document.hpp>
#include <anna/xml/Compiler.hpp>

namespace anna {

namespace xml {

/**
   Clase para gestionar un documento XML contenido en un buffer Cstring.
*/
class DocumentMemory : public Document {
public:
  /**
     Constructor.
  */
  DocumentMemory() : a_rootNode(NULL) {;}

  /**
  * Root node after parsing, or NULL if not analyzed by mean parse()
  *
  * @return XML document root node
  */
  const Node * getRootNode() const throw() { return a_rootNode; }

  /**
   * Parse xml document and return root node
   * \return Root node
   * \warning Need previous initialization
   */
  const Node* parse() throw(RuntimeException) { return (a_rootNode = Document::parse()); }

  /**
   * Parse xml document with dtd and return root node
   * \param dtd Validation DTD for XML document
   * \return Root node
   * \warning Need previous initialization
   */
  const Node* parse(const DTD& dtd) throw(RuntimeException) { return (a_rootNode = Document::parse(dtd)); }

  /**
   * XML representation from loaded document
   * \return XML string
   * \warning Need previous initialization and parsing
   */
  std::string asXmlString() const throw() { Compiler c; return (c.apply(a_rootNode)); }

private:
  const anna::xml::Node * a_rootNode;
  virtual _xmlDoc* do_initialize(const char* content) throw(RuntimeException);
  virtual _xmlDoc* do_initialize(const anna::DataBlock&) throw(RuntimeException);
};

}
}

#endif
