// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <ctype.h>
#include <stdio.h>

#include <anna/core/DataBlock.hpp>
#include <anna/core/RuntimeException.hpp>
#include <anna/core/functions.hpp>

using namespace std;
using namespace anna;

#define anna_call_extend(a) \
   if ((a_size + (a)) >= a_maxSize) extend ((a));

DataBlock::DataBlock(const char* buffer, const int size, const bool deepCopy)
throw(RuntimeException) :
  a_buffer(NULL),
  a_size(0),
  a_deepCopy(deepCopy),
  a_maxSize(0) {
  initialize(buffer, size);
}

DataBlock::DataBlock(const DataBlock& other)
throw(RuntimeException) :
  a_buffer(NULL),
  a_size(0),
  a_deepCopy(other.a_deepCopy),
  a_maxSize(0) {
  initialize(other.a_buffer, other.a_size);
}

DataBlock::~DataBlock() {
  if(a_deepCopy)
    delete [] a_buffer;
}

void DataBlock::initialize(const char* buffer, const int size)
throw(RuntimeException) {
  if(a_deepCopy == false) {
    a_buffer = const_cast <char*>(buffer);
    a_size = size;
  } else {
    if((a_maxSize = a_size = size) > 0) {
      if((a_buffer = new char [size]) != NULL)
        anna_memcpy(a_buffer, buffer, size);
      else
        throw RuntimeException("Insufficient memory", ANNA_FILE_LOCATION);
    } else
      a_buffer = NULL;
  }
}

void DataBlock::setSize(const int size)
throw(RuntimeException) {
//   if (size > a_maxSize) {
//      string msg ("DataBlock::setSize | MaxSize: ");
//      msg += functions::asString (a_maxSize);
//      msg += " | Size: ";
//      msg += functions::asString (size);
//      msg += " | Invalid size";
//      throw RuntimeException (msg, ANNA_FILE_LOCATION);
//   }
//
//   a_currentMessage.assign (a_buffer.getData () + a_offset, remainingSize);
//
// Sin embargo, el fetch() de nuestro clientSocket hace esto:
//   a_buffer.setup (a_data.getData () + a_offset, remainingSize);
//
// Y el setup no hace reserva, solo asocia (buffer y size)
  a_size = size;
}

void DataBlock::allocate(const int nbytes)
throw(RuntimeException) {
  if(nbytes > a_size) {
    if(a_maxSize == 0) {
      if(a_deepCopy == false)
        throw RuntimeException("Deep-Copy was not active", ANNA_FILE_LOCATION);

      if((a_buffer = new char [a_maxSize = nbytes]) == NULL)
        throw RuntimeException("Insufficient memory", ANNA_FILE_LOCATION);
    } else
      extend(nbytes - a_size);

    a_size = nbytes;
  }
}

void DataBlock::assign(const char* buffer, const int size)
throw(RuntimeException) {
  if(a_deepCopy == false) {
    a_buffer = const_cast <char*>(buffer);
    a_size = size;
  } else {
    delete a_buffer;
    a_maxSize = a_size = 0;
    initialize(buffer, size);
  }
}

DataBlock& DataBlock::operator = (const DataBlock & right)
throw(RuntimeException) {
  if(this != &right) {
    if(a_deepCopy) {
      a_size = 0;
      append(right.a_buffer, right.a_size);
    } else {
      a_buffer = right.a_buffer;
      a_size = right.a_size;
    }
  }

  return *this;
}

const char DataBlock::operator [](const int pos) const
throw(RuntimeException) {
  if(pos >= a_size) {
    std::string msg("Position: ");
    msg += functions::asString(pos);
    msg += " | Out of range [0,";
    msg += functions::asString(a_size);
    msg += ')';
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  return a_buffer [pos];
}

char& DataBlock::operator [](const int pos)
throw(RuntimeException) {
  if(pos >= a_size) {
    std::string msg("Position: ");
    msg += functions::asString(pos);
    msg += " | Out of range [0,";
    msg += functions::asString(a_size);
    msg += ')';
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  char* aux = const_cast <char*>(a_buffer);
  return aux[pos];
}

void DataBlock::append(const char* data, const int len)
throw(RuntimeException) {
  if(a_buffer == data && data != NULL && len > 0)
    throw RuntimeException("Can not append myself", ANNA_FILE_LOCATION);

  if((a_size + len) >= a_maxSize) extend(len);

  char* w = a_buffer + a_size;

  switch(len) {
  case 0: break;
  case 1: *w = *data; a_size ++; break;
  case 2: *w = *data; *(w + 1) = *(data + 1); a_size += 2; break;
  case 3: *w = *data; *(w + 1) = *(data + 1); *(w + 2) = *(data + 2); a_size += 3; break;
  case 4: *w = *data; *(w + 1) = *(data + 1); *(w + 2) = *(data + 2); *(w + 3) = *(data + 3); a_size += 4; break;
  default: anna_memcpy(w, data, len); a_size += len; break;
  }
}

void DataBlock::remove(const int pos, const int nbytes)
throw(RuntimeException) {
  if(a_deepCopy == false)
    throw RuntimeException("Deep copy not activated", ANNA_FILE_LOCATION);

  if(pos >= a_size || pos < 0)
    throw RuntimeException("Position to remove is out of range", ANNA_FILE_LOCATION);

  const int nbytesToMove = a_size - (pos + nbytes);

  if(nbytesToMove < 0)
    throw RuntimeException("nbytes to remove are out of range", ANNA_FILE_LOCATION);

  if(nbytesToMove > 0)
    anna_memmove(a_buffer + pos, a_buffer + pos + nbytes, nbytesToMove);

  a_size -= nbytes;
}

void DataBlock::remove(const int nbytes)
throw(RuntimeException) {
  if(a_deepCopy == false)
    throw RuntimeException("Deep copy not activated", ANNA_FILE_LOCATION);

  if(nbytes == 0)
    return;

  const int nbytesToMove = a_size - nbytes;

  if(nbytesToMove < 0)
    throw RuntimeException("nbytes to remove are out of range", ANNA_FILE_LOCATION);

  if(nbytesToMove > 0)
    anna_memmove(a_buffer, a_buffer + nbytes, nbytesToMove);

  a_size -= nbytes;
}

void DataBlock::extend(const int nbytes)
throw(RuntimeException) {
  if(a_deepCopy == false)
    throw RuntimeException("Deep copy not activated", ANNA_FILE_LOCATION);

  if((a_size + nbytes) < a_maxSize)
    return;

  int newSize = ((a_size + nbytes)  << 1) - ((a_size + nbytes)  >> 1);
  char* newBuffer = new char [newSize];

  if(newBuffer == NULL)
    throw RuntimeException("Insufficient memory", ANNA_FILE_LOCATION);

  if(a_size > 0)
    anna_memcpy(newBuffer, a_buffer, a_size);

  delete [] a_buffer;
  a_buffer = newBuffer;
  a_maxSize = newSize;
}

std::string DataBlock::asString(const int characterByLine) const
throw() {
  char aux [8];
  std::string numbers;
  std::string characters;
  unsigned char c;
  int i;
  std::string result;

  if(a_size == 0) {
    result = "<null>";
    return result;
  }

  result += " (";
  result += functions::asHexString(anna_ptrnumber_cast(a_buffer));
  result += "):";

  for(i = 0; i < a_size; i ++) {
    if((i % characterByLine) == 0) {
      if(i > 0) {
        result += '\n';
        result += numbers;
        result += "  ";
        result += characters;
        characters.clear();
      }

      sprintf(aux, "%4d: ", i);
      numbers = aux;
    }

    c = (unsigned char) a_buffer [i];
    sprintf(aux, "%02x ", c);
    numbers += aux;
    characters += (isprint(c) ? (char) c : '.');
  }

  result += '\n';
  result += numbers;

  while((i % characterByLine) != 0) {
    result += "   ";
    i ++;
  }

  result += "  ";
  return result += characters;
}
