// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/mt/Guard.hpp>
#include <anna/core/mt/Safe.hpp>
#include <anna/core/functions.hpp>
#include <anna/core/tracing/Logger.hpp>

using namespace std;
using namespace anna;

#define trace_local7(method,object,file,line) \
   if (Logger::isActive (Logger::Local7)) { \
      string msg = (method); \
      msg += " | Reference: "; \
      if (a_whatis == NULL)  \
         msg += functions::asHexString (anna_ptrnumber_cast ((object))); \
      else { \
         msg += "("; \
         msg += functions::asHexString (anna_ptrnumber_cast ((object))); \
         msg += "): "; \
         msg += a_whatis; \
      } \
      Logger::write (Logger::Local7, msg, (file), (line)); \
   }

Guard::Guard(const Safe* object) throw(RuntimeException) :
  a_whatis(NULL) {
  if(object == NULL)
    throw RuntimeException("Can not lock NULL object", ANNA_FILE_LOCATION);

  lock(a_safeObject = const_cast <Safe*>(object), NULL);
}

Guard::Guard(const Safe* object, const char* whatis) throw(RuntimeException) :
  a_whatis(whatis) {
  if(object == NULL)
    throw RuntimeException("Can not lock NULL object", ANNA_FILE_LOCATION);

  lock(a_safeObject = const_cast <Safe*>(object), whatis);
}

void Guard::deactivate()
throw() {
  if(a_safeObject == NULL)
    return;

  a_safeObject->unlock();
  trace_local7("Guard::deactivate", a_safeObject, __FILE__, __LINE__);
  a_safeObject = NULL;
}

//--------------------------------------------------------------------------------------------------------------
// Solo sacmos trazas cuando nos mandan un 'whatis', de otro quedamos un bucle infinito porque el
// Logger::writer tambien usa una Guard.
//--------------------------------------------------------------------------------------------------------------
void Guard::lock(Safe* safe, const char* whatis)
throw(RuntimeException) {
  trace_local7("Guard::lock", safe, __FILE__, __LINE__);
  safe->lock();
}

