// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <unistd.h>
#include <signal.h>

#include <anna/core/functions.hpp>
#include <anna/core/tracing/TraceMethod.hpp>
#include <anna/core/tracing/TraceFunction.hpp>
#include <anna/core/tracing/Logger.hpp>
#include <anna/core/mt/Runnable.hpp>
#include <anna/core/mt/Thread.hpp>
#include <anna/core/mt/ThreadManager.hpp>

using namespace std;
using namespace anna;

Thread::~Thread() {
#ifdef _MT

  if(isRunning() == true) {
    LOGWARNING(
      string msg(asString());
      msg += " | Destroying active thread";
      Logger::warning(msg, ANNA_FILE_LOCATION);
    );
//      pthread_cancel (a_id);
    a_id = (pthread_t) - 1;
  }

#endif
}

void Thread::start(Runnable& runnable)
throw(RuntimeException) {
  if(isRunning() == true) {
    std::string msg(asString());
    msg += " | Already activated";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  a_data.thread = this;
  a_data.runnable = &runnable;
#ifdef _MT
  pthread_attr_t attr;
  pthread_attr_init(&attr);

  if(isJoinable() == false)
    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);

  int errorCode;

  if((errorCode = pthread_create(&a_id, &attr, Thread::exec, &a_data)) != 0) {
    if(a_data.thread->a_manager != NULL)
      a_data.thread->a_manager->releaseThread(a_data.thread);

    throw RuntimeException(std::string("Thread::start"), errorCode, ANNA_FILE_LOCATION);
  }

#else
  exec(&a_data);
#endif
}

void Thread::join()
throw(RuntimeException) {
#ifdef _MT
  int errorCode;
  const pthread_t self(pthread_self());

  if(pthread_equal(a_id, self) != 0) {
    string msg("Thread::join | ");
    msg += asString();
    msg += " | Cannot call join from itself";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }

  if((errorCode = pthread_join(a_id, NULL)) != 0)
    throw RuntimeException(std::string("Thread::join"), errorCode, ANNA_FILE_LOCATION);

#endif
}

/* Cdigo correspondiente al thread */
void* Thread::exec(void* arg) {
  WHEN_MULTITHREAD(
    sigset_t mask;
    sigemptyset(&mask);
    pthread_sigmask(SIG_SETMASK, &mask, NULL);
  );
  Data* data = reinterpret_cast <Data*>(arg);
  LOGDEBUG(
    string msg("Starting thread | ");
    msg += data->thread->asString();
    msg += data->runnable->asString();
    Logger::debug(msg, ANNA_FILE_LOCATION);
  );

  try {
    data->runnable->initialize();
    data->runnable->setIsRunning(true);
    data->runnable->run();
    data->runnable->setIsRunning(false);
    LOGDEBUG(
      string msg("Finishing thread | ");
      msg += data->thread->asString();
      msg += data->runnable->asString();
      Logger::debug(msg, ANNA_FILE_LOCATION);
    );
    data->runnable->terminate();
  } catch(Exception& ex) {
    data->runnable->setIsRunning(false);
    LOGDEBUG(
      string msg("Finishing thread | ");
      msg += data->thread->asString();
      msg += data->runnable->asString();
      Logger::debug(msg, ANNA_FILE_LOCATION);
    );
    data->runnable->terminate();
    ex.trace();
  }

  data->thread->a_id = (pthread_t) - 1;

  if(data->thread->a_manager != NULL)
    data->thread->a_manager->releaseThread(data->thread);

  return NULL;
}

std::string Thread::asString() const
throw() {
  std::string result("anna::Thread { Id: ");
  result += functions::asHexString((int) a_id);
  result += functions::asString(" | Flags (0x%x): ", a_flags);
  result += (isJoinable()) ? "Joinable" : "None";
  result += " | Running: ";
  result += functions::asString(isRunning());
  result += " | ThreadManager: ";

  if(a_manager == NULL)
    result += "<none>";
  else
    result += a_manager->getName();

  return result += " }";
}

