// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/http/parser/WaitMessage.hpp>

#include <anna/http/internal/Token.hpp>
#include <anna/http/Transport.hpp>
#include <anna/http/Method.hpp>
#include <anna/http/Request.hpp>
#include <anna/http/Response.hpp>

using namespace std;
using namespace anna;

//----------------------------------------------------------------------------------------------------
// Todavia no se ha identificado el comienzo del mensaje HTTP. Comprueba el primer token de la
// linea para comprobar si se puede identificar una mensaje HTTP de tipo 'Response' o 'Request'.
//
// La expresion del mensaje HTTP sera:
//
// Mensahe HTTP := Request | Response
//
// Resquest := Method <sp> URI <sp> version <crlf> *(Header)<crlf><crlf> [Body]
// Response := version <sp> status_code <sp> phrase <crlf> *(Header)
//
// (1) Si recibimos una linea vacia o con solo CRLF's
// (2) Separa el contenido de la linea en los tokens. El separador seran espacios en blanco, tabs, etc.
// (3) Si estamos recibiendo un Request.
// (4) Si estamos recibiendo un Response
//----------------------------------------------------------------------------------------------------
int http::parser::WaitMessage::processLine(http::Transport& transport, const DataBlock&, const http::Token& line) const
throw(RuntimeException) {
  if(line.getSize() == 0)                                                         // (1)
    return -1;

  const Tokenizer& tokenizer = transport.split(line);                             // (2)
  const Token* token = tokenizer [tokenizer.begin()];

  if(token == NULL)
    return -1;

  if(Method::asType(token) != Method::Type::None)                                  // (3)
    if(setupRequest(transport, tokenizer) == true)
      return -1;

  if(token != NULL) {
    if(token->match("http/1.1") || token->match("http/1.0"))                      // (4)
      setupResponse(transport, tokenizer);
  }

  return -1;
}

//----------------------------------------------------------------------------------------------------
// (1) Verifica que en la linea solo esten los tres elementos esperados
//----------------------------------------------------------------------------------------------------
/*static*/
bool http::parser::WaitMessage::setupRequest(http::Transport& transport, const http::Tokenizer& tokenizer)
throw() {
  bool result = true;
  const Token* token;
  Tokenizer::const_iterator ii = tokenizer.begin();

  try {
    if((token = tokenizer [ii ++]) == NULL)
      return false;

    const Method::Type::_v method = Method::asType(token);

    if((token = tokenizer [ii ++]) == NULL)
      return false;

    const string& uri(token->getStringValue());

    if((token = tokenizer [ii ++]) == NULL)
      return false;

    if(tokenizer [ii] != NULL)                                           // (1)
      return false;

    const string& version(token->getStringValue());
    http::Request* request = static_cast <http::Request*>(transport.allocateInputMessage(Message::Type::Request));
    request->setMethod(method);
    request->setURI(uri);
    request->setVersion(version);
    setState(transport, ClassType::ReadHeader);
  } catch(RuntimeException&) {
    result = false;
  }

  return result;
}

//----------------------------------------------------------------------------------------------------
// (1) IN_NEMRD_01.00.00_http_001
//----------------------------------------------------------------------------------------------------
/*static*/
void http::parser::WaitMessage::setupResponse(http::Transport& transport, const http::Tokenizer& tokenizer)
throw() {
  const Token* token;
  Tokenizer::const_iterator ii = tokenizer.begin();

  try {
    if((token = tokenizer [ii ++]) == NULL)
      return;

    const string& version(token->getStringValue());

    if((token = tokenizer [ii ++]) == NULL)
      return;

    const int statusCode(token->getIntegerValue());
    string reasonPhrase;

    if((token = tokenizer [ii ++]) != NULL) {                               // (1)
      reasonPhrase = token->getStringValue();

      while((token = tokenizer [ii ++]) != NULL) {
        reasonPhrase += ' ';
        reasonPhrase += token->getStringValue();
      }
    }

    http::Response* response = static_cast <http::Response*>(transport.allocateInputMessage(Message::Type::Response));
    response->setVersion(version);
    response->setStatusCode(statusCode);
    response->setReasonPhrase(reasonPhrase);
    setState(transport, ClassType::ReadHeader);
  } catch(RuntimeException&) {
  }
}
