// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <stdio.h>

#include <anna/config/defines.hpp>

#include <anna/io/AbstractReader.hpp>
#include <anna/io/internal/sccs.hpp>

using namespace std;
using namespace anna;

io::AbstractReader::AbstractReader() :
  a_file(NULL),
  a_ex(NULL) {
  sccs::activate();
}

io::AbstractReader::AbstractReader(const char* filename) :
  a_filename(filename),
  a_ex(NULL) {
  if((a_file = fopen(filename, "r")) == NULL)
    a_ex = new RuntimeException(filename, errno, ANNA_FILE_LOCATION);

  sccs::activate();
}

io::AbstractReader::AbstractReader(const std::string& filename) :
  a_filename(filename),
  a_ex(NULL) {
  if((a_file = fopen(filename.c_str(), "r")) == NULL)
    a_ex = new RuntimeException(filename, errno, ANNA_FILE_LOCATION);

  sccs::activate();
}

/* virtual */
io::AbstractReader::~AbstractReader() {
  close();
}

void io::AbstractReader::close()
throw() {
  delete a_ex;
  a_ex = NULL;

  if(a_file != NULL) {
    fclose(a_file);
    a_file = NULL;
  }
}

void io::AbstractReader::open(const std::string& filename)
throw(RuntimeException) {
  close();

  if((a_file = fopen(filename.c_str(), "r")) == NULL)
    throw RuntimeException(filename, errno, ANNA_FILE_LOCATION);
}

void io::AbstractReader::verify()
throw(RuntimeException) {
  if(a_ex != NULL)
    throw *a_ex;

  if(a_file == NULL) {
    string msg("io::AbstractReader | File: ");
    msg += a_filename;
    msg += " | Cannot open";
    throw RuntimeException(msg, ANNA_FILE_LOCATION);
  }
}
